@@ -33,13 +33,13 @@
 
 #include <utils/algorithm.h>
 #include <utils/hostosinfo.h>
+#include <utils/macroexpander.h>
 #include <utils/qtcassert.h>
 #include <utils/qtcprocess.h>
 #include <utils/fancylineedit.h>
 
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/variablechooser.h>
-#include <coreplugin/variablemanager.h>
 
 #include <QTextStream>
 #include <QMimeData>
@@ -58,7 +58,6 @@ static const Qt::ItemFlags TOOL_ITEM_FLAGS = Qt::ItemIsSelectable | Qt::ItemIsEn
 ExternalToolModel::ExternalToolModel(QObject *parent)
     : QAbstractItemModel(parent)
 {
-    setSupportedDragActions(Qt::MoveAction);
 }
 
 ExternalToolModel::~ExternalToolModel()
@@ -411,10 +410,11 @@ ExternalToolConfig::ExternalToolConfig(QWidget *parent) :
     connect(ui->toolTree->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
             this, SLOT(handleCurrentChanged(QModelIndex,QModelIndex)));
 
-    Core::VariableChooser::addVariableSupport(ui->executable->lineEdit());
-    Core::VariableChooser::addVariableSupport(ui->arguments);
-    Core::VariableChooser::addVariableSupport(ui->workingDirectory->lineEdit());
-    Core::VariableChooser::addVariableSupport(ui->inputText);
+    auto chooser = new VariableChooser(this);
+    chooser->addSupportedWidget(ui->executable->lineEdit());
+    chooser->addSupportedWidget(ui->arguments);
+    chooser->addSupportedWidget(ui->workingDirectory->lineEdit());
+    chooser->addSupportedWidget(ui->inputText);
 
     connect(ui->description, SIGNAL(editingFinished()), this, SLOT(updateCurrentItem()));
     connect(ui->executable, SIGNAL(editingFinished()), this, SLOT(updateCurrentItem()));
@@ -442,7 +442,6 @@ ExternalToolConfig::ExternalToolConfig(QWidget *parent) :
 
     showInfoForItem(QModelIndex());
 
-    new VariableChooser(this);
 }
 
 ExternalToolConfig::~ExternalToolConfig()
@@ -522,11 +521,11 @@ void ExternalToolConfig::showInfoForItem(const QModelIndex &index)
     updateButtons(index);
     ExternalTool *tool = m_model->toolForIndex(index);
     if (!tool) {
-        ui->description->setText(QString());
+        ui->description->clear();
         ui->executable->setPath(QString());
-        ui->arguments->setText(QString());
+        ui->arguments->clear();
         ui->workingDirectory->setPath(QString());
-        ui->inputText->setPlainText(QString());
+        ui->inputText->clear();
         ui->infoWidget->setEnabled(false);
         return;
     }
@@ -595,6 +594,5 @@ void ExternalToolConfig::addCategory()
 
 void ExternalToolConfig::updateEffectiveArguments()
 {
-    ui->arguments->setToolTip(Utils::QtcProcess::expandMacros(ui->arguments->text(),
-            Core::VariableManager::macroExpander()));
+    ui->arguments->setToolTip(Utils::globalMacroExpander()->expandProcessArgs(ui->arguments->text()));
 }
