@@ -30,7 +30,7 @@
 
 #include "cppcodeformatter.h"
 
-#include <texteditor/basetextdocumentlayout.h>
+#include <texteditor/textdocumentlayout.h>
 #include <cplusplus/Lexer.h>
 
 #include <utils/qtcassert.h>
@@ -81,7 +81,7 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
         m_tokenIndex = 1;
     }
 
-    for (; m_tokenIndex < m_tokens.size(); ) {
+    while (m_tokenIndex < m_tokens.size()) {
         m_currentToken = tokenAt(m_tokenIndex);
         const int kind = m_currentToken.kind();
 
@@ -460,7 +460,7 @@ void CodeFormatter::recalculateStateAfter(const QTextBlock &block)
             if (kind != T_CASE && kind != T_DEFAULT && tryStatement())
                 break;
             switch (kind) {
-            case T_RBRACE:      leave(); continue;
+            case T_RBRACE:
             case T_DEFAULT:
             case T_CASE:        leave(); continue;
             } break;
@@ -1091,7 +1091,7 @@ int CodeFormatter::tokenizeBlock(const QTextBlock &block, bool *endedJoined)
         *endedJoined = tokenize.endedJoined();
 
     const int lexerState = tokenize.state();
-    BaseTextDocumentLayout::setLexerState(block, lexerState);
+    TextDocumentLayout::setLexerState(block, lexerState);
     return lexerState;
 }
 
@@ -1144,7 +1144,7 @@ void QtStyleCodeFormatter::setCodeStyleSettings(const CppCodeStyleSettings &sett
 
 void QtStyleCodeFormatter::saveBlockData(QTextBlock *block, const BlockData &data) const
 {
-    TextBlockUserData *userData = BaseTextDocumentLayout::userData(*block);
+    TextBlockUserData *userData = TextDocumentLayout::userData(*block);
     CppCodeFormatterData *cppData = static_cast<CppCodeFormatterData *>(userData->codeFormatterData());
     if (!cppData) {
         cppData = new CppCodeFormatterData;
@@ -1155,7 +1155,7 @@ void QtStyleCodeFormatter::saveBlockData(QTextBlock *block, const BlockData &dat
 
 bool QtStyleCodeFormatter::loadBlockData(const QTextBlock &block, BlockData *data) const
 {
-    TextBlockUserData *userData = BaseTextDocumentLayout::testUserData(block);
+    TextBlockUserData *userData = TextDocumentLayout::testUserData(block);
     if (!userData)
         return false;
     CppCodeFormatterData *cppData = static_cast<CppCodeFormatterData *>(userData->codeFormatterData());
@@ -1168,12 +1168,12 @@ bool QtStyleCodeFormatter::loadBlockData(const QTextBlock &block, BlockData *dat
 
 void QtStyleCodeFormatter::saveLexerState(QTextBlock *block, int state) const
 {
-    BaseTextDocumentLayout::setLexerState(*block, state);
+    TextDocumentLayout::setLexerState(*block, state);
 }
 
 int QtStyleCodeFormatter::loadLexerState(const QTextBlock &block) const
 {
-    return BaseTextDocumentLayout::lexerState(block);
+    return TextDocumentLayout::lexerState(block);
 }
 
 void QtStyleCodeFormatter::addContinuationIndent(int *paddingDepth) const
@@ -1449,7 +1449,7 @@ void QtStyleCodeFormatter::onEnter(int newState, int *indentDepth, int *savedInd
     *savedPaddingDepth = qMax(0, *savedPaddingDepth);
 }
 
-void QtStyleCodeFormatter::adjustIndent(const QList<CPlusPlus::Token> &tokens, int lexerState, int *indentDepth, int *paddingDepth) const
+void QtStyleCodeFormatter::adjustIndent(const Tokens &tokens, int lexerState, int *indentDepth, int *paddingDepth) const
 {
     State topState = state();
     State previousState = state(1);
@@ -1601,8 +1601,6 @@ void QtStyleCodeFormatter::adjustIndent(const QList<CPlusPlus::Token> &tokens, i
             } else if (type == case_cont) {
                 *indentDepth = state(i).savedIndentDepth;
                 break;
-            } else if (type == topmost_intro) {
-                break;
             }
         }
         break;
@@ -1668,6 +1666,7 @@ void QtStyleCodeFormatter::adjustIndent(const QList<CPlusPlus::Token> &tokens, i
             if (m_styleSettings.indentControlFlowRelativeToSwitchLabels)
                 *indentDepth += m_tabSettings.m_indentSize;
         }
+        break;
     }
     // ensure padding and indent are >= 0
     *indentDepth = qMax(0, *indentDepth);
