@@ -10,16 +10,17 @@
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** conditions see http://www.qt.io/licensing.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
 ** rights.  These rights are described in the Digia Qt LGPL Exception
@@ -126,6 +127,7 @@ WatchData::WatchData() :
     size(0),
     bitpos(0),
     bitsize(0),
+    elided(0),
     hasChildren(false),
     valueEnabled(true),
     valueEditable(true),
@@ -142,12 +144,12 @@ bool WatchData::isEqual(const WatchData &other) const
       && name == other.name
       && value == other.value
       && editvalue == other.editvalue
-      && valuetooltip == other.valuetooltip
       && type == other.type
       && displayedType == other.displayedType
       && variable == other.variable
       && address == other.address
       && size == other.size
+      && elided == other.elided
       && hasChildren == other.hasChildren
       && valueEnabled == other.valueEnabled
       && valueEditable == other.valueEditable
@@ -312,6 +314,9 @@ QString WatchData::toString() const
     if (isValueKnown() && !value.isEmpty())
         str << "value=\"" << value << doubleQuoteComma;
 
+    if (elided)
+        str << "valueelided=\"" << elided << doubleQuoteComma;
+
     if (!editvalue.isEmpty())
         str << "editvalue=\"<...>\",";
     //    str << "editvalue=\"" << editvalue << doubleQuoteComma;
@@ -337,36 +342,11 @@ QString WatchData::toString() const
     return res + QLatin1Char('}');
 }
 
-static QString htmlEscape(const QString &plain)
-{
-#if QT_VERSION >= 0x050000
-    return Qt::escape(plain);
-#else
-    // Copied from Qt to avoid GUI dependency
-    // (Qt::escape has been moved in Qt 5)
-    QString rich;
-    rich.reserve(int(plain.length() * qreal(1.1)));
-    for (int i = 0; i < plain.length(); ++i) {
-        if (plain.at(i) == QLatin1Char('<'))
-            rich += QLatin1String("&lt;");
-        else if (plain.at(i) == QLatin1Char('>'))
-            rich += QLatin1String("&gt;");
-        else if (plain.at(i) == QLatin1Char('&'))
-            rich += QLatin1String("&amp;");
-        else if (plain.at(i) == QLatin1Char('"'))
-            rich += QLatin1String("&quot;");
-        else
-            rich += plain.at(i);
-    }
-    return rich;
-#endif
-}
-
 // Format a tooltip fow with aligned colon.
 static void formatToolTipRow(QTextStream &str,
     const QString &category, const QString &value)
 {
-    QString val = htmlEscape(value);
+    QString val = value.toHtmlEscaped();
     val.replace(QLatin1Char('\n'), QLatin1String("<br>"));
     str << "<tr><td>" << category << "</td><td> : </td><td>"
         << val << "</td></tr>";
@@ -382,7 +362,7 @@ QString WatchData::toToolTip() const
     formatToolTipRow(str, tr("Internal Type"), QLatin1String(type));
     if (!displayedType.isEmpty())
         formatToolTipRow(str, tr("Displayed Type"), displayedType);
-    QString val = valuetooltip.isEmpty() ? value : valuetooltip;
+    QString val = value;
     // Automatically display hex value for unsigned integers.
     if (!val.isEmpty() && val.at(0).isDigit() && isIntType(type)) {
         bool ok;
@@ -458,13 +438,6 @@ void WatchData::updateValue(const GdbMi &item)
     }
 }
 
-void setWatchDataValueToolTip(WatchData &data, const GdbMi &mi,
-    int encoding)
-{
-    if (mi.isValid())
-        data.valuetooltip = decodeData(mi.data(), encoding);
-}
-
 void WatchData::updateChildCount(const GdbMi &mi)
 {
     if (mi.isValid())
@@ -541,6 +514,16 @@ void WatchData::updateDisplayedType(const GdbMi &item)
 
 // Utilities to decode string data returned by the dumper helpers.
 
+template <class T>
+QString decodeItemHelper(const T &t)
+{
+    return QString::number(t);
+}
+
+QString decodeItemHelper(const double &t)
+{
+    return QString::number(t, 'g', 16);
+}
 
 template <class T>
 void decodeArrayHelper(QList<WatchData> *list, const WatchData &tmplate,
@@ -555,7 +538,7 @@ void decodeArrayHelper(QList<WatchData> *list, const WatchData &tmplate,
         data.sortId = i;
         data.iname += QByteArray::number(i);
         data.name = QString::fromLatin1("[%1]").arg(i);
-        data.value = QString::number(p[i]);
+        data.value = decodeItemHelper(p[i]);
         data.address += i * sizeof(T);
         data.exp = exp + QByteArray::number(data.address, 16);
         data.setAllUnneeded();
@@ -622,13 +605,16 @@ void parseWatchData(const QSet<QByteArray> &expandedINames,
         data.editvalue = mi.data();
 
     mi = item["editformat"];
-    if (mi.isValid())
-        data.editformat = mi.toInt();
+    data.editformat = mi.toInt();
 
     mi = item["typeformats"];
     if (mi.isValid())
         data.typeFormats = QString::fromUtf8(mi.data());
 
+    mi = item["valueelided"];
+    if (mi.isValid())
+        data.elided = mi.toInt();
+
     mi = item["bitpos"];
     if (mi.isValid())
         data.bitpos = mi.toInt();
