@@ -39,6 +39,7 @@
 #include "projectexplorerconstants.h"
 #include "toolchainmanager.h"
 
+#include <utils/algorithm.h>
 #include <utils/detailswidget.h>
 #include <utils/environment.h>
 #include <utils/pathchooser.h>
@@ -219,17 +220,14 @@ IOutputParser *CustomToolChain::outputParser() const
 
 QStringList CustomToolChain::headerPathsList() const
 {
-    QStringList list;
-    foreach (const HeaderPath &headerPath, m_systemHeaderPaths)
-        list << headerPath.path();
-    return list;
+    return Utils::transform(m_systemHeaderPaths, &HeaderPath::path);
 }
 
 void CustomToolChain::setHeaderPaths(const QStringList &list)
 {
-    m_systemHeaderPaths.clear();
-    foreach (const QString &headerPath, list)
-        m_systemHeaderPaths << HeaderPath(headerPath.trimmed(), HeaderPath::GlobalHeaderPath);
+    m_systemHeaderPaths = Utils::transform(list, [](const QString &headerPath) {
+        return HeaderPath(headerPath.trimmed(), HeaderPath::GlobalHeaderPath);
+    });
 }
 
 void CustomToolChain::setCompilerCommand(const FileName &path)
@@ -273,9 +271,8 @@ const QStringList &CustomToolChain::cxx11Flags() const
 
 void CustomToolChain::setMkspecs(const QString &specs)
 {
-    m_mkspecs.clear();
-    foreach (const QString &spec, specs.split(QLatin1Char(',')))
-        m_mkspecs << FileName::fromString(spec);
+    m_mkspecs = Utils::transform(specs.split(QLatin1Char(',')),
+                                 [](QString fn) { return Utils::FileName::fromString(fn); });
 }
 
 QString CustomToolChain::mkspecs() const
@@ -590,9 +587,9 @@ void CustomToolChainConfigWidget::setFromToolchain()
     m_compilerCommand->setFileName(tc->compilerCommand());
     m_makeCommand->setFileName(FileName::fromString(tc->makeCommand(Utils::Environment())));
     m_abiWidget->setAbis(QList<Abi>(), tc->targetAbi());
-    m_predefinedMacros->setPlainText(tc->rawPredefinedMacros().join(QLatin1String("\n")));
-    m_headerPaths->setPlainText(tc->headerPathsList().join(QLatin1String("\n")));
-    m_cxx11Flags->setText(tc->cxx11Flags().join(QLatin1String(",")));
+    m_predefinedMacros->setPlainText(tc->rawPredefinedMacros().join(QLatin1Char('\n')));
+    m_headerPaths->setPlainText(tc->headerPathsList().join(QLatin1Char('\n')));
+    m_cxx11Flags->setText(tc->cxx11Flags().join(QLatin1Char(',')));
     m_mkspecs->setText(tc->mkspecs());
     m_errorParserComboBox->setCurrentIndex(tc->outputParserType());
     m_customParserSettings = tc->customParserSettings();
