@@ -76,6 +76,7 @@ public:
     explicit DotRemovalFilter(QObject *parent = 0);
 protected:
     virtual bool filterAcceptsRow(int source_row, const QModelIndex &parent) const;
+    Qt::DropActions supportedDragActions() const;
 };
 
 DotRemovalFilter::DotRemovalFilter(QObject *parent) : QSortFilterProxyModel(parent)
@@ -91,12 +92,18 @@ bool DotRemovalFilter::filterAcceptsRow(int source_row, const QModelIndex &paren
     return fileName != QLatin1String(".");
 }
 
+Qt::DropActions DotRemovalFilter::supportedDragActions() const
+{
+    return sourceModel()->supportedDragActions();
+}
+
 // FolderNavigationModel: Shows path as tooltip.
 class FolderNavigationModel : public QFileSystemModel
 {
 public:
     explicit FolderNavigationModel(QObject *parent = 0);
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
+    Qt::DropActions supportedDragActions() const;
 };
 
 FolderNavigationModel::FolderNavigationModel(QObject *parent) :
@@ -112,6 +119,11 @@ QVariant FolderNavigationModel::data(const QModelIndex &index, int role) const
         return QFileSystemModel::data(index, role);
 }
 
+Qt::DropActions FolderNavigationModel::supportedDragActions() const
+{
+    return Qt::MoveAction;
+}
+
 /*!
   \class FolderNavigationWidget
 
@@ -142,6 +154,8 @@ FolderNavigationWidget::FolderNavigationWidget(QWidget *parent)
     m_listView->setModel(m_filterModel);
     m_listView->setFrameStyle(QFrame::NoFrame);
     m_listView->setAttribute(Qt::WA_MacShowFocusRect, false);
+    m_listView->setDragEnabled(true);
+    m_listView->setDragDropMode(QAbstractItemView::DragOnly);
     setFocusProxy(m_listView);
 
     QVBoxLayout *layout = new QVBoxLayout();
@@ -416,30 +430,10 @@ void FolderNavigationWidget::ensureCurrentIndex()
 // --------------------FolderNavigationWidgetFactory
 FolderNavigationWidgetFactory::FolderNavigationWidgetFactory()
 {
-}
-
-FolderNavigationWidgetFactory::~FolderNavigationWidgetFactory()
-{
-}
-
-QString FolderNavigationWidgetFactory::displayName() const
-{
-    return tr("File System");
-}
-
-int FolderNavigationWidgetFactory::priority() const
-{
-    return 400;
-}
-
-Core::Id FolderNavigationWidgetFactory::id() const
-{
-    return "File System";
-}
-
-QKeySequence FolderNavigationWidgetFactory::activationSequence() const
-{
-    return QKeySequence(Core::UseMacShortcuts ? tr("Meta+Y") : tr("Alt+Y"));
+    setDisplayName(tr("File System"));
+    setPriority(400);
+    setId("File System");
+    setActivationSequence(QKeySequence(Core::UseMacShortcuts ? tr("Meta+Y") : tr("Alt+Y")));
 }
 
 Core::NavigationView FolderNavigationWidgetFactory::createWidget()
