@@ -10,16 +10,17 @@
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** conditions see http://www.qt.io/licensing.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
 ** rights.  These rights are described in the Digia Qt LGPL Exception
@@ -31,6 +32,7 @@
 #define REMOTELINUXRUNCONFIGURATION_H
 
 #include "remotelinux_export.h"
+#include "abstractremotelinuxrunconfiguration.h"
 
 #include <projectexplorer/runconfiguration.h>
 
@@ -50,7 +52,7 @@ class RemoteLinuxRunConfigurationPrivate;
 class RemoteLinuxRunConfigurationFactory;
 } // namespace Internal
 
-class REMOTELINUX_EXPORT RemoteLinuxRunConfiguration : public ProjectExplorer::RunConfiguration
+class REMOTELINUX_EXPORT RemoteLinuxRunConfiguration : public AbstractRemoteLinuxRunConfiguration
 {
     Q_OBJECT
     Q_DISABLE_COPY(RemoteLinuxRunConfiguration)
@@ -58,8 +60,8 @@ class REMOTELINUX_EXPORT RemoteLinuxRunConfiguration : public ProjectExplorer::R
     friend class RemoteLinuxRunConfigurationWidget;
 
 public:
-    RemoteLinuxRunConfiguration(ProjectExplorer::Target *parent, const Core::Id id,
-        const QString &projectFilePath);
+    RemoteLinuxRunConfiguration(ProjectExplorer::Target *parent, Core::Id id,
+        const QString &targetName);
     ~RemoteLinuxRunConfiguration();
 
     bool isEnabled() const;
@@ -82,10 +84,6 @@ public:
 
     QVariantMap toMap() const;
 
-    int portsUsedByDebuggers() const;
-
-    QString projectFilePath() const;
-
     static const char *IdPrefix;
 
 signals:
