@@ -10,16 +10,17 @@
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** conditions see http://www.qt.io/licensing.  For further information
+** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
 ** rights.  These rights are described in the Digia Qt LGPL Exception
@@ -33,6 +34,7 @@
 #include "buildgraph.h"
 #include "command.h"
 #include "cycledetector.h"
+#include "emptydirectoriesremover.h"
 #include "productbuilddata.h"
 #include "projectbuilddata.h"
 #include "rulesevaluationcontext.h"
@@ -44,7 +46,6 @@
 #include <tools/persistence.h>
 #include <tools/propertyfinder.h>
 #include <tools/qbsassert.h>
-#include <tools/setupprojectparameters.h>
 
 #include <QDir>
 #include <QFileInfo>
@@ -98,15 +99,38 @@ static void restoreBackPointers(const ResolvedProjectPtr &project)
     }
 }
 
-BuildGraphLoadResult BuildGraphLoader::load(const SetupProjectParameters &parameters,
-        const RulesEvaluationContextPtr &evalContext)
+BuildGraphLoadResult BuildGraphLoader::load(const TopLevelProjectPtr &existingProject,
+                                            const SetupProjectParameters &parameters,
+                                            const RulesEvaluationContextPtr &evalContext)
 {
+    m_parameters = parameters;
     m_result = BuildGraphLoadResult();
     m_evalContext = evalContext;
 
-    const QString projectId = TopLevelProject::deriveId(parameters.buildConfigurationTree());
+    if (existingProject) {
+        QBS_CHECK(existingProject->buildData);
+        existingProject->buildData->evaluationContext = evalContext;
+        checkBuildGraphCompatibility(existingProject);
+        m_result.loadedProject = existingProject;
+    } else {
+        loadBuildGraphFromDisk();
+    }
+    if (!m_result.loadedProject)
+        return m_result;
+    if (parameters.restoreBehavior() == SetupProjectParameters::RestoreOnly)
+        return m_result;
+    QBS_CHECK(parameters.restoreBehavior() == SetupProjectParameters::RestoreAndTrackChanges);
+
+    trackProjectChanges();
+    return m_result;
+}
+
+void BuildGraphLoader::loadBuildGraphFromDisk()
+{
+    const QString projectId = TopLevelProject::deriveId(m_parameters.topLevelProfile(),
+                                                        m_parameters.finalBuildConfigurationTree());
     const QString buildDir
-            = TopLevelProject::deriveBuildDirectory(parameters.buildRoot(), projectId);
+            = TopLevelProject::deriveBuildDirectory(m_parameters.buildRoot(), projectId);
     const QString buildGraphFilePath
             = ProjectBuildData::deriveBuildGraphFilePath(buildDir, projectId);
 
@@ -115,27 +139,39 @@ BuildGraphLoadResult BuildGraphLoader::load(const SetupProjectParameters &parame
     try {
         pool.load(buildGraphFilePath);
     } catch (const ErrorInfo &loadError) {
-        if (parameters.restoreBehavior() == SetupProjectParameters::RestoreOnly)
+        if (m_parameters.restoreBehavior() == SetupProjectParameters::RestoreOnly)
             throw;
         m_logger.qbsInfo() << loadError.toString();
-        return m_result;
+        return;
     }
 
     const TopLevelProjectPtr project = TopLevelProject::create();
 
     // TODO: Store some meta data that will enable us to show actual progress (e.g. number of products).
-    evalContext->initializeObserver(Tr::tr("Restoring build graph from disk"), 1);
+    m_evalContext->initializeObserver(Tr::tr("Restoring build graph from disk"), 1);
 
     project->load(pool);
-    project->buildData->evaluationContext = evalContext;
+    project->buildData->evaluationContext = m_evalContext;
+    project->setBuildConfiguration(pool.headData().projectConfig);
+    project->buildDirectory = buildDir;
+    checkBuildGraphCompatibility(project);
+    restoreBackPointers(project);
+    project->location = CodeLocation(m_parameters.projectFilePath(), project->location.line(),
+                                     project->location.column());
+    m_result.loadedProject = project;
+    m_evalContext->incrementProgressValue();
+    doSanityChecks(project, m_logger);
+}
 
-    if (QFileInfo(project->location.fileName()) != QFileInfo(parameters.projectFilePath())) {
+void BuildGraphLoader::checkBuildGraphCompatibility(const TopLevelProjectConstPtr &project)
+{
+    if (QFileInfo(project->location.fileName()) != QFileInfo(m_parameters.projectFilePath())) {
         QString errorMessage = Tr::tr("Stored build graph at '%1' is for project file '%2', but "
                                       "input file is '%3'. ")
-                .arg(QDir::toNativeSeparators(buildGraphFilePath),
+                .arg(QDir::toNativeSeparators(project->buildGraphFilePath()),
                      QDir::toNativeSeparators(project->location.fileName()),
-                     QDir::toNativeSeparators(parameters.projectFilePath()));
-        if (!parameters.ignoreDifferentProjectFilePath()) {
+                     QDir::toNativeSeparators(m_parameters.projectFilePath()));
+        if (!m_parameters.ignoreDifferentProjectFilePath()) {
             errorMessage += Tr::tr("Aborting.");
             throw ErrorInfo(errorMessage);
         }
@@ -144,38 +180,21 @@ BuildGraphLoadResult BuildGraphLoader::load(const SetupProjectParameters &parame
         errorMessage += Tr::tr("Ignoring.");
         m_logger.qbsWarning() << errorMessage;
     }
-
-    restoreBackPointers(project);
-
-    project->location = CodeLocation(parameters.projectFilePath(), project->location.line(),
-                                     project->location.column());
-    project->setBuildConfiguration(pool.headData().projectConfig);
-    project->buildDirectory = buildDir;
-    m_result.loadedProject = project;
-    evalContext->incrementProgressValue();
-    doSanityChecks(project, m_logger);
-
-    if (parameters.restoreBehavior() == SetupProjectParameters::RestoreOnly)
-        return m_result;
-    QBS_CHECK(parameters.restoreBehavior() == SetupProjectParameters::RestoreAndTrackChanges);
-
-    trackProjectChanges(parameters, buildGraphFilePath, project, pool.headData().projectConfig);
-    return m_result;
 }
 
-void BuildGraphLoader::trackProjectChanges(const SetupProjectParameters &parameters,
-        const QString &buildGraphFilePath, const TopLevelProjectPtr &restoredProject,
-        const QVariantMap &oldProjectConfig)
+void BuildGraphLoader::trackProjectChanges()
 {
-    const FileTime buildGraphTimeStamp = FileInfo(buildGraphFilePath).lastModified();
+    const TopLevelProjectPtr &restoredProject = m_result.loadedProject;
     QSet<QString> buildSystemFiles = restoredProject->buildSystemFiles;
     QList<ResolvedProductPtr> allRestoredProducts = restoredProject->allProducts();
     QList<ResolvedProductPtr> changedProducts;
     QList<ResolvedProductPtr> productsWithChangedFiles;
     bool reResolvingNecessary = false;
-    if (!isConfigCompatible(parameters.finalBuildConfigurationTree(), oldProjectConfig))
+    if (!isConfigCompatible(m_parameters.finalBuildConfigurationTree(),
+                            restoredProject->buildConfiguration())) {
         reResolvingNecessary = true;
-    if (hasProductFileChanged(allRestoredProducts, buildGraphTimeStamp,
+    }
+    if (hasProductFileChanged(allRestoredProducts, restoredProject->lastResolveTime,
                               buildSystemFiles, productsWithChangedFiles)) {
         reResolvingNecessary = true;
     }
@@ -184,7 +203,7 @@ void BuildGraphLoader::trackProjectChanges(const SetupProjectParameters &paramet
     // can make the list of source files in a product change without the respective file
     // having been touched. In such a case, the build data for that product will have to be set up
     // anew.
-    if (hasBuildSystemFileChanged(buildSystemFiles, buildGraphTimeStamp)
+    if (hasBuildSystemFileChanged(buildSystemFiles, restoredProject->lastResolveTime)
             || hasEnvironmentChanged(restoredProject)
             || hasFileExistsResultChanged(restoredProject)
             || hasFileLastModifiedResultChanged(restoredProject)) {
@@ -196,15 +215,15 @@ void BuildGraphLoader::trackProjectChanges(const SetupProjectParameters &paramet
 
     restoredProject->buildData->isDirty = true;
     Loader ldr(m_evalContext->engine(), m_logger);
-    ldr.setSearchPaths(parameters.searchPaths());
+    ldr.setSearchPaths(m_parameters.searchPaths());
     ldr.setProgressObserver(m_evalContext->observer());
-    m_result.newlyResolvedProject = ldr.loadProject(parameters);
+    m_result.newlyResolvedProject = ldr.loadProject(m_parameters);
 
     QMap<QString, ResolvedProductPtr> freshProductsByName;
     QList<ResolvedProductPtr> allNewlyResolvedProducts
             = m_result.newlyResolvedProject->allProducts();
     foreach (const ResolvedProductPtr &cp, allNewlyResolvedProducts)
-        freshProductsByName.insert(cp->name, cp);
+        freshProductsByName.insert(cp->uniqueName(), cp);
 
     checkAllProductsForChanges(allRestoredProducts, freshProductsByName, changedProducts,
                                productsWithChangedFiles);
@@ -231,11 +250,16 @@ void BuildGraphLoader::trackProjectChanges(const SetupProjectParameters &paramet
     // For products with "serious" changes such as different prepare scripts, we set up the
     // build data from scratch to be on the safe side. This can be made more fine-grained
     // if needed.
+    QHash<QString, AllRescuableArtifactData> rescuableArtifactData;
     foreach (const ResolvedProductPtr &product, changedProducts) {
-        ResolvedProductPtr freshProduct = freshProductsByName.value(product->name);
+        ResolvedProductPtr freshProduct = freshProductsByName.value(product->uniqueName());
         if (!freshProduct)
             continue;
         onProductRemoved(product, product->topLevelProject()->buildData.data(), false);
+        if (product->buildData) {
+            rescuableArtifactData.insert(product->uniqueName(),
+                                         product->buildData->rescuableArtifactData);
+        }
         allRestoredProducts.removeOne(product);
         productsWithChangedFiles.removeOne(product);
     }
@@ -248,22 +272,11 @@ void BuildGraphLoader::trackProjectChanges(const SetupProjectParameters &paramet
         const ResolvedProductPtr &newlyResolvedProduct = allNewlyResolvedProducts.at(i);
         for (int j = allRestoredProducts.count() - 1; j >= 0; --j) {
             const ResolvedProductPtr &restoredProduct = allRestoredProducts.at(j);
-            if (newlyResolvedProduct->name == restoredProduct->name) {
-                if (newlyResolvedProduct->enabled) {
+            if (newlyResolvedProduct->uniqueName() == restoredProduct->uniqueName()) {
+                if (newlyResolvedProduct->enabled)
                     newlyResolvedProduct->buildData.swap(restoredProduct->buildData);
-                } else {
-                    if (restoredProduct->enabled) {
-                        QBS_CHECK(restoredProduct->buildData);
-                        foreach (Artifact * const a,
-                                ArtifactSet::fromNodeSet(restoredProduct->buildData->nodes)) {
-                            const bool removeFromDisk = a->artifactType == Artifact::Generated;
-                            newlyResolvedProduct->topLevelProject()->buildData->removeArtifact(a,
-                                    m_logger, removeFromDisk, true);
-                            m_objectsToDelete << a;
-                        }
-                    }
+                else
                     productsWithChangedFiles.removeOne(restoredProduct);
-                }
                 if (newlyResolvedProduct->buildData) {
                     foreach (BuildGraphNode *node, newlyResolvedProduct->buildData->nodes)
                         node->product = newlyResolvedProduct;
@@ -296,17 +309,26 @@ void BuildGraphLoader::trackProjectChanges(const SetupProjectParameters &paramet
     // For products where only the list of files has changed, we adapt the existing build data
     // so we won't recompile existing files just because new ones have been added.
     foreach (const ResolvedProductPtr &product, productsWithChangedFiles) {
-        ResolvedProductPtr freshProduct = freshProductsByName.value(product->name);
+        ResolvedProductPtr freshProduct = freshProductsByName.value(product->uniqueName());
         if (!freshProduct)
             continue;
         onProductFileListChanged(product, freshProduct, oldBuildData.data());
     }
 
     foreach (const ResolvedProductConstPtr &changedProduct, changedProducts) {
-        rescueOldBuildData(changedProduct, freshProductsByName.value(changedProduct->name),
-                           oldBuildData.data(), childLists);
+        rescueOldBuildData(changedProduct, freshProductsByName.value(changedProduct->uniqueName()),
+                           oldBuildData.data(), childLists,
+                           rescuableArtifactData.value(changedProduct->uniqueName()));
     }
 
+    EmptyDirectoriesRemover(m_result.newlyResolvedProject.data(), m_logger)
+            .removeEmptyParentDirectories(m_artifactsRemovedFromDisk);
+
+    foreach (FileResourceBase * const f, m_objectsToDelete) {
+        Artifact * const a = dynamic_cast<Artifact *>(f);
+        if (a)
+            a->product.clear(); // To help with the sanity checks.
+    }
     doSanityChecks(m_result.newlyResolvedProject, m_logger);
 }
 
@@ -314,9 +336,12 @@ bool BuildGraphLoader::hasEnvironmentChanged(const TopLevelProjectConstPtr &rest
 {
     for (QHash<QString, QString>::ConstIterator it = restoredProject->usedEnvironment.constBegin();
          it != restoredProject->usedEnvironment.constEnd(); ++it) {
-        if (m_environment.value(it.key()) != it.value()) {
-            m_logger.qbsDebug() << "A relevant environment variable changed, "
-                                   "must re-resolve project.";
+        const QString var = it.key();
+        const QString oldValue = it.value();
+        const QString newValue = m_environment.value(var);
+        if (newValue != oldValue) {
+            m_logger.qbsDebug() << QString::fromLatin1("Environment variable '%1' changed "
+                "from '%2' to '%3'. Must re-resolve project.").arg(var, oldValue, newValue);
             return true;
         }
     }
@@ -410,18 +435,25 @@ void BuildGraphLoader::checkAllProductsForChanges(const QList<ResolvedProductPtr
         if (changedProducts.contains(restoredProduct))
             continue;
         const ResolvedProductPtr newlyResolvedProduct
-                = newlyResolvedProductsByName.value(restoredProduct->name);
+                = newlyResolvedProductsByName.value(restoredProduct->uniqueName());
         if (!newlyResolvedProduct)
             continue;
+        if (newlyResolvedProduct->enabled != restoredProduct->enabled) {
+            m_logger.qbsDebug() << "Condition of product '" << restoredProduct->uniqueName()
+                                << "' was changed, must set up build data from scratch";
+            changedProducts << restoredProduct;
+            continue;
+        }
+
         if (!productsWithChangedFiles.contains(restoredProduct)
                 && !sourceArtifactSetsAreEqual(restoredProduct->allFiles(),
                                                 newlyResolvedProduct->allFiles())) {
-            m_logger.qbsDebug() << "File list of product '" << restoredProduct->name
+            m_logger.qbsDebug() << "File list of product '" << restoredProduct->uniqueName()
                                 << "' was changed.";
             productsWithChangedFiles += restoredProduct;
         }
         if (checkProductForChanges(restoredProduct, newlyResolvedProduct)) {
-            m_logger.qbsDebug() << "Product '" << restoredProduct->name
+            m_logger.qbsDebug() << "Product '" << restoredProduct->uniqueName()
                                 << "' was changed, must set up build data from scratch";
             changedProducts << restoredProduct;
         }
@@ -436,9 +468,9 @@ static bool dependenciesAreEqual(const ResolvedProductConstPtr &p1,
     QSet<QString> names1;
     QSet<QString> names2;
     foreach (const ResolvedProductConstPtr &dep, p1->dependencies)
-        names1 << dep->name;
+        names1 << dep->uniqueName();
     foreach (const ResolvedProductConstPtr &dep, p2->dependencies)
-        names2 << dep->name;
+        names2 << dep->uniqueName();
     return names1 == names2;
 }
 
@@ -480,7 +512,7 @@ bool BuildGraphLoader::checkForPropertyChanges(const ResolvedProductPtr &restore
                                                const ResolvedProductPtr &newlyResolvedProduct)
 {
     m_logger.qbsDebug() << "Checking for changes in properties requested in prepare scripts for "
-                           "product '"  << restoredProduct->name << "'.";
+                           "product '"  << restoredProduct->uniqueName() << "'.";
     if (!restoredProduct->buildData)
         return false;
 
@@ -488,6 +520,12 @@ bool BuildGraphLoader::checkForPropertyChanges(const ResolvedProductPtr &restore
     if (checkTransformersForPropertyChanges(restoredProduct, newlyResolvedProduct))
         return true;
 
+    if (restoredProduct->fileTags != newlyResolvedProduct->fileTags) {
+        m_logger.qbsTrace() << "Product type changed from " << restoredProduct->fileTags
+                            << "to " << newlyResolvedProduct->fileTags;
+        return true;
+    }
+
     if (checkProductForInstallInfoChanges(restoredProduct, newlyResolvedProduct))
         return true;
     if (!artifactPropertyListsAreEqual(restoredProduct->artifactProperties,
@@ -512,7 +550,7 @@ bool BuildGraphLoader::checkTransformersForPropertyChanges(const ResolvedProduct
     }
     if (transformerChanges) {
         m_logger.qbsDebug() << "Property changes in product '"
-                            << newlyResolvedProduct->name << "'.";
+                            << newlyResolvedProduct->uniqueName() << "'.";
     }
     return transformerChanges;
 }
@@ -520,23 +558,36 @@ bool BuildGraphLoader::checkTransformersForPropertyChanges(const ResolvedProduct
 void BuildGraphLoader::onProductRemoved(const ResolvedProductPtr &product,
         ProjectBuildData *projectBuildData, bool removeArtifactsFromDisk)
 {
-    m_logger.qbsDebug() << "[BG] product '" << product->name << "' removed.";
+    m_logger.qbsDebug() << "[BG] product '" << product->uniqueName() << "' removed.";
 
     product->project->products.removeOne(product);
     if (product->buildData) {
-        foreach (Artifact *artifact, ArtifactSet::fromNodeSet(product->buildData->nodes))
-            projectBuildData->removeArtifact(artifact, m_logger, removeArtifactsFromDisk, false);
+        foreach (BuildGraphNode * const node, product->buildData->nodes) {
+            if (node->type() == BuildGraphNode::ArtifactNodeType) {
+                Artifact * const artifact = static_cast<Artifact *>(node);
+                projectBuildData->removeArtifact(artifact, m_logger, removeArtifactsFromDisk,
+                                                 false);
+                if (removeArtifactsFromDisk && artifact->artifactType == Artifact::Generated)
+                    m_artifactsRemovedFromDisk << artifact->filePath();
+            } else {
+                foreach (BuildGraphNode * const parent, node->parents)
+                    parent->children.remove(node);
+                node->parents.clear();
+                foreach (BuildGraphNode * const child, node->children)
+                    child->parents.remove(node);
+                node->children.clear();
+            }
+        }
     }
 }
 
 void BuildGraphLoader::onProductFileListChanged(const ResolvedProductPtr &restoredProduct,
         const ResolvedProductPtr &newlyResolvedProduct, const ProjectBuildData *oldBuildData)
 {
-    m_logger.qbsDebug() << "[BG] product '" << restoredProduct->name << "' changed.";
+    m_logger.qbsDebug() << "[BG] product '" << restoredProduct->uniqueName() << "' changed.";
 
     QBS_CHECK(newlyResolvedProduct->enabled);
 
-    QList<Artifact *> addedArtifacts;
     ArtifactSet artifactsToRemove;
     QHash<QString, SourceArtifactConstPtr> oldArtifacts, newArtifacts;
 
@@ -548,7 +599,7 @@ void BuildGraphLoader::onProductFileListChanged(const ResolvedProductPtr &restor
         if (!oldArtifacts.contains(a->absoluteFilePath)) {
             // artifact added
             m_logger.qbsDebug() << "[BG] artifact '" << a->absoluteFilePath
-                                << "' added to product " << restoredProduct->name;
+                                << "' added to product " << restoredProduct->uniqueName();
             Artifact *newArtifact = lookupArtifact(newlyResolvedProduct, oldBuildData,
                                                    a->absoluteFilePath, true);
             if (newArtifact) {
@@ -575,7 +626,6 @@ void BuildGraphLoader::onProductFileListChanged(const ResolvedProductPtr &restor
                                                       newArtifact);
                 }
             }
-            addedArtifacts += newArtifact;
         }
     }
 
@@ -584,7 +634,7 @@ void BuildGraphLoader::onProductFileListChanged(const ResolvedProductPtr &restor
         if (!changedArtifact) {
             // artifact removed
             m_logger.qbsDebug() << "[BG] artifact '" << a->absoluteFilePath
-                                << "' removed from product " << restoredProduct->name;
+                                << "' removed from product " << restoredProduct->uniqueName();
             Artifact *artifact
                     = lookupArtifact(restoredProduct, oldBuildData, a->absoluteFilePath, true);
             QBS_CHECK(artifact);
@@ -603,18 +653,13 @@ void BuildGraphLoader::onProductFileListChanged(const ResolvedProductPtr &restor
             Artifact *artifact
                     = lookupArtifact(restoredProduct, oldBuildData, a->absoluteFilePath, true);
             QBS_CHECK(artifact);
-
             // handle added filetags
-            foreach (const FileTag &addedFileTag, changedArtifact->fileTags - a->fileTags) {
-                artifact->fileTags += addedFileTag;
-                newlyResolvedProduct->registerAddedFileTag(addedFileTag, artifact);
-            }
+            foreach (const FileTag &addedFileTag, changedArtifact->fileTags - a->fileTags)
+                artifact->addFileTag(addedFileTag);
 
             // handle removed filetags
-            foreach (const FileTag &removedFileTag, a->fileTags - changedArtifact->fileTags) {
-                artifact->fileTags -= removedFileTag;
-                newlyResolvedProduct->registerRemovedFileTag(removedFileTag, artifact);
-            }
+            foreach (const FileTag &removedFileTag, a->fileTags - changedArtifact->fileTags)
+                artifact->removeFileTag(removedFileTag);
         }
 
         if (changedArtifact->properties->value() != a->properties->value()) {
@@ -626,15 +671,16 @@ void BuildGraphLoader::onProductFileListChanged(const ResolvedProductPtr &restor
             newlyResolvedProduct->topLevelProject()->buildData
                 ->removeArtifactAndExclusiveDependents(oldArtifact, m_logger, true,
                                                        &artifactsToRemove);
-            addedArtifacts += createArtifact(newlyResolvedProduct, changedArtifact, m_logger);
+            createArtifact(newlyResolvedProduct, changedArtifact, m_logger);
         }
     }
 
     // defer destruction of removed artifacts
-    foreach (Artifact *artifact, artifactsToRemove)
+    foreach (Artifact *artifact, artifactsToRemove) {
+        if (artifact->artifactType == Artifact::Generated)
+            m_artifactsRemovedFromDisk << artifact->filePath();
         m_objectsToDelete << artifact;
-    foreach (Artifact * const artifact, addedArtifacts)
-        newlyResolvedProduct->registerAddedArtifact(artifact);
+    }
 }
 
 static SourceArtifactConstPtr findSourceArtifact(const ResolvedProductConstPtr &product,
@@ -758,15 +804,18 @@ void BuildGraphLoader::replaceFileDependencyWithArtifact(const ResolvedProductPt
 
 void BuildGraphLoader::rescueOldBuildData(const ResolvedProductConstPtr &restoredProduct,
         const ResolvedProductPtr &newlyResolvedProduct, ProjectBuildData *oldBuildData,
-        const ChildListHash &childLists)
+        const ChildListHash &childLists, const AllRescuableArtifactData &existingRad)
 {
     if (!restoredProduct->enabled || !newlyResolvedProduct->enabled)
         return;
 
     if (m_logger.traceEnabled()) {
         m_logger.qbsTrace() << QString::fromLocal8Bit("[BG] rescue data of "
-                                                      "product '%1'").arg(restoredProduct->name);
+                "product '%1'").arg(restoredProduct->uniqueName());
     }
+    QBS_CHECK(newlyResolvedProduct->buildData);
+    QBS_CHECK(newlyResolvedProduct->buildData->rescuableArtifactData.isEmpty());
+    newlyResolvedProduct->buildData->rescuableArtifactData = existingRad;
 
     // This is needed for artifacts created by manually added transformers, which are
     // already present in the build graph.
@@ -796,6 +845,7 @@ void BuildGraphLoader::rescueOldBuildData(const ResolvedProductConstPtr &restore
             if (m_logger.traceEnabled())
                 m_logger.qbsTrace() << QString::fromLocal8Bit("[BG]    artifact invalidated");
             removeGeneratedArtifactFromDisk(oldArtifact, m_logger);
+            m_artifactsRemovedFromDisk << oldArtifact->filePath();
             continue;
         }
         artifact->setTimestamp(oldArtifact->timestamp());
@@ -828,7 +878,8 @@ void BuildGraphLoader::rescueOldBuildData(const ResolvedProductConstPtr &restore
             const ChildrenInfo &childrenInfo = childLists.value(oldArtifact);
             foreach (Artifact * const child, childrenInfo.children) {
                 rad.children << RescuableArtifactData::ChildData(child->product->name,
-                        child->filePath(), childrenInfo.childrenAddedByScanner.contains(child));
+                        child->product->profile, child->filePath(),
+                        childrenInfo.childrenAddedByScanner.contains(child));
             }
             newlyResolvedProduct->buildData->rescuableArtifactData.insert(
                         oldArtifact->filePath(), rad);
